{-# LANGUAGE FlexibleContexts #-}
{-# LANGUAGE DeriveGeneric #-}
{-# LANGUAGE DuplicateRecordFields #-}

module Espresso.Config where

import           Control.Monad.IO.Class         ( MonadIO
                                                , liftIO
                                                )
import           GHC.Generics                   ( Generic )

import           Control.Exception.Lifted       ( Exception
                                                , throwIO
                                                )
import           Control.Monad.Base             ( MonadBase )
import           System.Directory               ( getCurrentDirectory )
import           Data.Aeson                     ( FromJSON )
import           Data.Yaml                      ( decodeFileEither
                                                , prettyPrintParseException
                                                )
import           Network.Wai.Serve.Types        ( StageConfiguration )
import           Data.Text                      ( Text )
import qualified Data.Text                     as T

data ConfigException = ConfigParseException Text deriving (Show)

instance Exception ConfigException

data Config = Config
  { server :: StageConfiguration
  , bundle :: BundleConfig
  } deriving (Generic)

instance FromJSON Config

data BundleConfig
  = S3Bundle S3BundleConfig
  | LocalBundle LocalBundleConfig
  deriving (Generic, Show)

instance FromJSON BundleConfig

data S3BundleConfig = S3BundleConfig
  { accessKey :: Text
  , secretKey :: Text
  , endpoint :: Text
  , bucket :: Text
  , objectName :: Text
  , pollSeconds :: Int
  } deriving (Generic, Show)

instance FromJSON S3BundleConfig

data LocalBundleConfig = LocalBundleConfig
  { path :: Text
  } deriving (Generic, Show)

instance FromJSON LocalBundleConfig

loadConfig :: (MonadIO m, MonadBase IO m) => m Config
loadConfig = do
  configFile <- (<> "/config.yaml") <$> liftIO getCurrentDirectory
  parseResult <- liftIO $ decodeFileEither configFile

  case parseResult of
    Left parserError -> throwIO
      $ ConfigParseException (T.pack $ prettyPrintParseException parserError)
    Right decodedConfig -> pure decodedConfig
